<?php
// Update redirected links in your website
// (c) 2017-2021 Scriptol.com
// Free under the MIT License.
// Requires the PHP 7 interpreter.
// Works on a local copy of your site.
// Searches for all 301 redirected links
// and replaces them by the target URL.
// Once all pages updated, you have to put them online
// with a script such as PHP FTP Synchronizer.
//
include_once("scriptol-path.php");
include_once("scriptol-dom.php");
$pcounter=0;
$dcounter=0;
$lcounter=0;
$mcounter=0;
$bcounter=0;
$content="";
$website="";
$root="";
$rootlen=0;
$DISPONLY=false;
$VERBOSE=false;
$RECONSTRUCT=false;
// Add your extension if not in list
$extensions=array(".html",".htm",".asp",".php",".https");
function usage()
{
   echo "\n";
   echo "UnRedir - (c) 2017-2021 Scriptol.com", "\n";
   echo "Usage:", "\n";
   echo "Go to the root of the local copy of the website and type:", "\n";
   echo "  php unredir.php [option][domain-name]", "\n";
   echo "Options:", "\n";
   echo "  domain  To build a full URL from a relative path. Replaced only if redirected.", "\n";
   echo "  -t      Display changes to be done, does not modify the files.", "\n";
   echo "  -v      Verbose, display each link checked.", "\n";
   exit(0);
   return;
}

// Utilities
function hasProtocol($url)
{
   if(strpos($url,"://")===false)
   {
      return false;
   }
   return true;
}

function rebuildURL($url,$locdir)
{
   if($url==="")
   {
      return "";
   }
   if(substr($url,0,3)==="../")
   {
      return $url;
   }
   if($url[0]==="/")
   {
      global $website;
      return "//".$website.substr($url,1);
   }
   global $rootlen;
   if(strlen($locdir)>=$rootlen)
   {
      $locdir=substr($locdir,$rootlen);
   }
   global $website;
   $locdir="http://".$website.$locdir;
   return Path::merge($locdir,$url);
}

function dispBroken($url)
{
   echo " *** Broken:", " ", $url, "\n";
   global $bcounter;
   $bcounter+=1;
   return "";
}

// Check for redirect
function redirected($url)
{
   global $VERBOSE;
   if($VERBOSE)
   {
      echo "Check", " ", $url, "\n";
   }
   if(strlen($url)<8)
   {
      return "";
   }
   $code=0;
   $newurl="";
   $hcurl=curl_init();
   
    curl_setopt($hcurl, CURLOPT_CONNECTTIMEOUT, 300);
    curl_setopt($hcurl, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($hcurl, CURLOPT_VERBOSE, false);
    curl_setopt($hcurl, CURLOPT_URL, $url);
    curl_setopt($hcurl, CURLOPT_HEADER, true);
    curl_setopt($hcurl, CURLOPT_NOBODY, true);
    curl_setopt($hcurl, CURLOPT_FOLLOWLOCATION, false);
    curl_setopt($hcurl, CURLOPT_SSL_VERIFYPEER, false);
    $headers = curl_exec($hcurl);
    $code = curl_getinfo($hcurl, CURLINFO_HTTP_CODE);

   if($code===404)
   {
      curl_close($hcurl);
      return "404";
   }
   if($code!=301)
   {
      curl_close($hcurl);
      return "";
   }
   
    curl_setopt($hcurl, CURLOPT_FOLLOWLOCATION, true);
    $headers = curl_exec($hcurl);
    $newurl = curl_getinfo($hcurl, CURLINFO_EFFECTIVE_URL);
    $code = curl_getinfo($hcurl, CURLINFO_HTTP_CODE);

   curl_close($hcurl);
   if($code===404)
   {
      return "404";
   }
   if($code!=200)
   {
      return "";
   }
   return $newurl;
}

// Display page path
$dflag=true;
function dispPage($fpath)
{
   global $dflag;
   if($dflag===false)
   {
      return;
   }
   global $VERBOSE;
   if(!$VERBOSE)
   {
      echo ".", "\n";
   }
   echo $fpath, "\n";
   echo str_repeat("-",strlen($fpath)), "\n";
   $dflag=false;
   return;
}

// Search and replace a relative path (reg exp to be avoided)
function relativeReplace($link,$nlink,$attribute)
{
   $ctr=0;
   $ll=strlen($link);
   global $content;
   $ptr=intVal(strpos($content,$link));
   if($ptr===false)
   {
      return 0;
   }
   if($ptr===0)
   {
      return 0;
   }
   if(($ptr+$ll)>=strlen($content))
   {
      return 0;
   }
   $c=$content[$ptr-1];
   if(($c!='"')&&($c!="'"))
   {
      return 0;
   }
   $c2=$content[$ptr+$ll];
   if($c2!=$c)
   {
      return 0;
   }
   $oldstr=$c.$link.$c;
   $newstr=$c.$nlink.$c;
   $content=str_replace($oldstr,$newstr,$content,$ctr);
   return $ctr;
}

// Scan page for URLs
function linkProcess($link,$locdir,$fpath,$attribute)
{
   $rcount=0;
   $nlink="";
   if($link==="")
   {
      return 0;
   }
   if($link[0]==="#")
   {
      return 0;
   }
   if($link[0]===".")
   {
      return 0;
   }
   $chklink=$link;
   if(!hasProtocol($link))
   {
      global $RECONSTRUCT;
      if(!$RECONSTRUCT)
      {
         return 0;
      }
      $chklink=rebuildURL($link,$locdir);
   }
   $nlink=redirected($chklink);
   if($nlink==="")
   {
      return 0;
   }
   dispPage($fpath);
   if($nlink==="404")
   {
      dispBroken($link);
      return 0;
   }
   if($link===$chklink)
   {
      global $content;
      $content=str_replace($link,$nlink,$content,$rcount);
   }
   else
   {
      $rcount=relativeReplace($link,$nlink,$attribute);
   }
   if($rcount===0)
   {
      return 0;
   }
   global $VERBOSE;
   if(!$VERBOSE)
   {
      echo $link, "\n";
   }
   echo " Redirected to:", " ", $nlink, "\n";
   return $rcount;
}

function scanPage($fpath,$locdir)
{
   $dom=new DOMDocument("UTF-8");
   $node=null;
   $count=0;
   $link="";
   global $dflag;
   $dflag=true;

   global $VERBOSE;
   if($VERBOSE)
   {
      echo "Page :", " ", $fpath, "\n";
   }
   global $content;
   $content=file_get_contents($fpath);
   @
   $dom->loadHTML($content);
   global $pcounter;
   $pcounter+=1;
   foreach($dom->getElementsByTagName('a') as $node)
   {
      $link=$node->getAttribute("href");
      $count+=linkProcess($link,$locdir,$fpath,"href");
   }
   foreach($dom->getElementsByTagName('img') as $node)
   {
      $link=$node->getAttribute("src");
      $count+=linkProcess($link,$locdir,$fpath,"src");
   }
   if($count>0)
   {
      $putctr=false;
      global $DISPONLY;
      if(!$DISPONLY)
      {
         $putctr=file_put_contents($fpath,$content);
      }
      if($putctr!=false)
      {
         echo "Saved with ",$count," change",($count>1?"s":"");
         echo "\n";
      }
      else
      {
         echo "Not saved", "\n";
      }
      global $lcounter;
      $lcounter+=$count;
      global $mcounter;
      $mcounter+=1;
   }
   else
   {
      if(!$VERBOSE)
      {
         echo ".";
      }
   }
   return;
}

// Scan directory and subdirectories
function scanDirectory($locdir)
{
   echo "\n";
   echo $locdir, "\n";
   echo str_repeat("-",strlen($locdir)), "\n";
   $dirlist=scandir($locdir);
   if(empty($dirlist))
   {
      return;
   }
   // Processing files

   foreach($dirlist as $name)
   {
      if($name[0]===".")
      {
         continue;
      }
      $e=Path::getExtension($name);
      global $extensions;
      if(!(in_array($e,$extensions)))
      {
         continue;
      }
      $name=Path::merge($locdir,$name);
      if(filetype($name)==="file")
      {
         scanPage($name,$locdir);
         global $pcounter;
         $pcounter+=1;
      }
   }
   // Processing subdirs

   foreach($dirlist as $name)
   {
      if($name[0]==='.')
      {
         continue;
      }
      $name=Path::merge($locdir,$name);
      if(filetype($name)==="dir")
      {
         scanDirectory($name);
         global $dcounter;
         $dcounter+=1;
      }
   }
   return;
}

// Main program
function main($argc,$argv)
{
   $x=array_slice($argv,1);
   foreach($x as $param)
   {
      if($param==="-v")
      {
         global $VERBOSE;
         $VERBOSE=true;
         continue;
      }
      if($param==="-t")
      {
         global $DISPONLY;
         $DISPONLY=true;
         continue;
      }
      if($param[0]==="-")
      {
         echo "Unknown command $param", "\n";
         usage();
      }
      global $website;
      $website=$param;
      if($website[-1]!="/")
      {
         $website.="/";
      }
      global $RECONSTRUCT;
      $RECONSTRUCT=true;
   }
   if(!function_exists("curl_init"))
   {
      die("Curl extension must be enabled.");
   }
   global $root;
   $root=getcwd();
   global $rootlen;
   $rootlen=strlen($root);
   scanDirectory($root);
   global $pcounter;
   global $dcounter;
   echo $pcounter," page",($pcounter>1?"s":"")," scanned in ",$dcounter," dir",($dcounter>1?"s":""),", ";
   global $lcounter;
   echo $lcounter," link",($lcounter>1?"s":"")," redirected.";
   global $bcounter;
   if($bcounter>0)
   {
      echo "\n";
      echo $bcounter," broken link",($bcounter>1?"s":""),".";
   }
   echo "\n";
   global $mcounter;
   echo $mcounter," page",($mcounter>1?"s":"")," updated.";
   echo "\n";
   return 0;
}

main(intVal($argc),$argv);

?>
